@@ -3,7 +3,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2015 David R. MacIver
+# Most of this work is copyright (C) 2013-2016 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -36,6 +36,7 @@ from hypothesis._settings import Phase, Verbosity, HealthCheck
 from hypothesis.executors import new_style_executor, \
     default_new_style_executor
 from hypothesis.reporting import report, verbose_report, current_verbosity
+from hypothesis.statistics import note_engine_for_statistics
 from hypothesis.internal.compat import getargspec, str_to_bytes
 from hypothesis.internal.reflection import nicerepr, arg_string, \
     impersonate, copy_argspec, function_digest, fully_qualified_name, \
@@ -93,7 +94,7 @@ def reify_and_execute(
     def run(data):
         from hypothesis.control import note
 
-        with BuildContext(is_final=is_final):
+        with BuildContext(data, is_final=is_final):
             seed = data.draw(random_module()).seed
             if seed != 0:
                 note('random.seed(%d)' % (seed,))
@@ -112,13 +113,14 @@ def reify_and_execute(
 
 
 def seed(seed):
-    """
-    seed: Start the test execution from a specific seed. May be any hashable
-          object. No exact meaning for seed is provided other than that
-          for a fixed seed value Hypothesis will try the same actions (insofar
-          as it can given external sources of non-determinism. e.g. timing and
-          hash randomization).
-          Overrides the derandomize setting if it is present.
+    """seed: Start the test execution from a specific seed.
+
+    May be any hashable object. No exact meaning for seed is provided
+    other than that for a fixed seed value Hypothesis will try the same
+    actions (insofar as it can given external sources of non-
+    determinism. e.g. timing and hash randomization). Overrides the
+    derandomize setting if it is present.
+
     """
 
     def accept(test):
@@ -267,12 +269,13 @@ def given(*generator_arguments, **generator_kwargs):
                     test.__name__, arg_string(test, arguments, example_kwargs)
                 )
                 try:
-                    with BuildContext() as b:
+                    with BuildContext(None) as b:
                         test_runner(
                             None,
                             lambda data: test(*arguments, **example_kwargs)
                         )
                 except BaseException:
+                    traceback.print_exc()
                     report(message_on_failure)
                     for n in b.notes:
                         report(n)
@@ -471,6 +474,7 @@ def given(*generator_arguments, **generator_kwargs):
                 database_key=database_key,
             )
             runner.run()
+            note_engine_for_statistics(runner)
             run_time = time.time() - start_time
             timed_out = (
                 settings.timeout > 0 and
@@ -579,6 +583,7 @@ def find(specifier, condition, settings=None, random=None, database_key=None):
             'Expected SearchStrategy but got %r of type %s' % (
                 specifier, type(specifier).__name__
             ))
+    specifier.validate()
 
     search = specifier
 
@@ -587,7 +592,7 @@ def find(specifier, condition, settings=None, random=None, database_key=None):
     last_data = [None]
 
     def template_condition(data):
-        with BuildContext():
+        with BuildContext(data):
             try:
                 data.is_find = True
                 result = data.draw(search)
@@ -625,10 +630,12 @@ def find(specifier, condition, settings=None, random=None, database_key=None):
         database_key=database_key,
     )
     runner.run()
+    note_engine_for_statistics(runner)
     run_time = time.time() - start
     if runner.last_data.status == Status.INTERESTING:
-        with BuildContext():
-            return TestData.for_buffer(runner.last_data.buffer).draw(search)
+        data = TestData.for_buffer(runner.last_data.buffer)
+        with BuildContext(data):
+            return data.draw(search)
     if runner.valid_examples <= settings.min_satisfying_examples:
         if settings.timeout > 0 and run_time > settings.timeout:
             raise Timeout((
