@@ -3,7 +3,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2015 David R. MacIver
+# Most of this work is copyright (C) 2013-2016 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -24,7 +24,7 @@ import tempfile
 import unicodedata
 
 from hypothesis.configuration import tmpdir, storage_directory
-from hypothesis.internal.compat import hunichr, GzipFile
+from hypothesis.internal.compat import hunichr, GzipFile, FileExistsError
 
 
 def charmap_file():
@@ -41,10 +41,10 @@ def charmap():
     if _charmap is None:
         f = charmap_file()
         if not os.path.exists(f):
-            _charmap = {}
+            tmp_charmap = {}
             for i in range(0, sys.maxunicode + 1):
                 cat = unicodedata.category(hunichr(i))
-                rs = _charmap.setdefault(cat, [])
+                rs = tmp_charmap.setdefault(cat, [])
                 if rs and rs[-1][-1] == i - 1:
                     rs[-1][-1] += 1
                 else:
@@ -53,15 +53,19 @@ def charmap():
             # a stable format for our charmap.
             data = sorted(
                 (k, tuple((map(tuple, v))))
-                for k, v in _charmap.items())
+                for k, v in tmp_charmap.items())
 
             # Write the Unicode table atomically
             fd, tmpfile = tempfile.mkstemp(dir=tmpdir())
             os.close(fd)
             with GzipFile(tmpfile, 'wb', mtime=1) as o:
                 o.write(pickle.dumps(data, pickle.HIGHEST_PROTOCOL))
-            os.rename(tmpfile, f)
-
+            try:
+                os.rename(tmpfile, f)
+            except FileExistsError:  # pragma: no cover
+                # This exception is only raised on Windows, and coverage is
+                # measured on Linux.
+                pass
         with GzipFile(f, 'rb') as i:
             _charmap = dict(pickle.loads(i.read()))
     assert _charmap is not None
