@@ -3,7 +3,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2015 David R. MacIver
+# Most of this work is copyright (C) 2013-2016 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -19,6 +19,8 @@ from __future__ import division, print_function, absolute_import
 
 import math
 from decimal import Decimal
+from numbers import Rational
+from fractions import Fraction
 
 from hypothesis.errors import InvalidArgument
 from hypothesis.control import assume
@@ -160,8 +162,9 @@ def one_of(*args):
     """Return a strategy which generates values from any of the argument
     strategies.
 
-    This may be called with one iterable argument instead of multiple strategy
-    arguments. In which case one_of(x) and one_of(\*x) are equivalent.
+    This may be called with one iterable argument instead of multiple
+    strategy arguments. In which case one_of(x) and one_of(\*x) are
+    equivalent.
 
     """
     if len(args) == 1 and not isinstance(args[0], SearchStrategy):
@@ -188,41 +191,53 @@ def integers(min_value=None, max_value=None):
     """Returns a strategy which generates integers (in Python 2 these may be
     ints or longs).
 
-    If min_value is not None then all values will be >=
-    min_value. If max_value is not None then all values will be <= max_value
+    If min_value is not None then all values will be >= min_value. If
+    max_value is not None then all values will be <= max_value
 
     """
 
-    check_valid_integer(min_value)
-    check_valid_integer(max_value)
+    check_valid_bound(min_value, 'min_value')
+    check_valid_bound(max_value, 'max_value')
     check_valid_interval(min_value, max_value, 'min_value', 'max_value')
 
     from hypothesis.searchstrategy.numbers import IntegersFromStrategy, \
         BoundedIntStrategy, WideRangeIntStrategy
 
-    if min_value is None:
-        if max_value is None:
+    min_int_value = None
+    if min_value is not None:
+        min_int_value = int(min_value)
+        if min_int_value != min_value and min_value > 0:
+            min_int_value += 1
+
+    max_int_value = None
+    if max_value is not None:
+        max_int_value = int(max_value)
+        if max_int_value != max_value and max_value < 0:
+            max_int_value -= 1
+
+    if min_int_value is None:
+        if max_int_value is None:
             return (
                 WideRangeIntStrategy()
             )
         else:
-            return IntegersFromStrategy(0).map(lambda x: max_value - x)
+            return IntegersFromStrategy(0).map(lambda x: max_int_value - x)
     else:
-        if max_value is None:
-            return IntegersFromStrategy(min_value)
+        if max_int_value is None:
+            return IntegersFromStrategy(min_int_value)
         else:
-            assert min_value <= max_value
-            if min_value == max_value:
-                return just(min_value)
-            elif min_value >= 0:
-                return BoundedIntStrategy(min_value, max_value)
-            elif max_value <= 0:
-                return BoundedIntStrategy(-max_value, -min_value).map(
-                    lambda t: -t
-                )
+            assert min_int_value <= max_int_value
+            if min_int_value == max_int_value:
+                return just(min_int_value)
+            elif min_int_value >= 0:
+                return BoundedIntStrategy(min_int_value, max_int_value)
+            elif max_int_value <= 0:
+                return BoundedIntStrategy(
+                    -max_int_value, -min_int_value
+                ).map(lambda t: -t)
             else:
-                return integers(min_value=0, max_value=max_value) | \
-                    integers(min_value=min_value, max_value=0)
+                return integers(min_value=0, max_value=max_int_value) | \
+                    integers(min_value=min_int_value, max_value=0)
 
 
 @cacheable
@@ -388,7 +403,7 @@ def sampled_from(elements):
     elements.
 
     Note that as with just, values will not be copied and thus you
-    should be careful of using mutable data
+    should be careful of using mutable data.
 
     """
 
@@ -727,37 +742,14 @@ def random_module():
     )
 
 
-@cacheable
-@defines_strategy
-def fractions():
-    """Generates instances of fractions.Fraction."""
-    from fractions import Fraction
-    return tuples(integers(), integers(min_value=1)).map(
-        lambda t: Fraction(*t)
-    )
-
-
-@cacheable
-@defines_strategy
-def decimals():
-    """Generates instances of decimals.Decimal."""
-    return (
-        floats().map(float_to_decimal) |
-        fractions().map(
-            lambda f: Decimal(f.numerator) / f.denominator
-        )
-    )
-
-
 @cacheable
 @defines_strategy
 def builds(target, *args, **kwargs):
     """Generates values by drawing from args and kwargs and passing them to
     target in the appropriate argument position.
 
-    e.g. builds(target,
-    integers(), flag=booleans()) would draw an integer i and a boolean b and
-    call target(i, flag=b).
+    e.g. builds(target, integers(), flag=booleans()) would draw an
+    integer i and a boolean b and call target(i, flag=b).
 
     """
     return tuples(tuples(*args), fixed_dictionaries(kwargs)).map(
@@ -765,10 +757,74 @@ def builds(target, *args, **kwargs):
     )
 
 
+@cacheable
 @defines_strategy
-def recursive(base, extend, max_leaves=100):
+def fractions(min_value=None, max_value=None, max_denominator=None):
+    """Returns a strategy which generates Fractions.
+
+    If min_value is not None then all generated values are no less than
+    min_value.
+
+    If max_value is not None then all generated values are no greater than
+    max_value.
+
+    If max_denominator is not None then the absolute value of the denominator
+    of any generated values is no greater than max_denominator. Note that
+    max_denominator must be at least 1.
+
     """
-    base: A strategy to start from.
+    check_valid_bound(min_value, 'min_value')
+    check_valid_bound(max_value, 'max_value')
+    check_valid_interval(min_value, max_value, 'min_value', 'max_value')
+
+    check_valid_integer(max_denominator)
+    if max_denominator is not None and max_denominator < 1:
+        raise InvalidArgument(
+            u'Invalid denominator bound %s' % max_denominator
+        )
+
+    denominator_strategy = integers(min_value=1, max_value=max_denominator)
+
+    def dm_func(denom):
+        max_num = max_value * denom if max_value is not None else None
+        min_num = min_value * denom if min_value is not None else None
+
+        return builds(
+            Fraction,
+            integers(min_value=min_num, max_value=max_num),
+            just(denom)
+        )
+
+    return denominator_strategy.flatmap(dm_func)
+
+
+@cacheable
+@defines_strategy
+def decimals(min_value=None, max_value=None):
+    """Generates instances of decimals.Decimal.
+
+    If min_value is not None then all generated values are no less than
+    min_value.
+
+    If max_value is not None then all generated values are no greater than
+    max_value.
+
+    """
+    check_valid_bound(min_value, 'min_value')
+    check_valid_bound(max_value, 'max_value')
+    check_valid_interval(min_value, max_value, 'min_value', 'max_value')
+
+    floats_strategy = floats(min_value=min_value, max_value=max_value)
+    fractions_strategy = fractions(min_value=min_value, max_value=max_value)
+
+    return (
+        floats_strategy.map(float_to_decimal) |
+        fractions_strategy.map(lambda f: Decimal(f.numerator) / f.denominator)
+    )
+
+
+def recursive(base, extend, max_leaves=100):
+    """base: A strategy to start from.
 
     extend: A function which takes a strategy and returns a new strategy.
 
@@ -783,15 +839,9 @@ def recursive(base, extend, max_leaves=100):
     strategy that may return arbitrarily nested and mixed lists of booleans.
     So e.g. False, [True], [False, []], [[[[True]]]], are all valid values to
     be drawn from that strategy.
+
     """
 
-    check_strategy(base)
-    extended = extend(base)
-    if not isinstance(extended, SearchStrategy):
-        raise InvalidArgument(
-            'Expected extend(%r) to be a SearchStrategy but got %r' % (
-                base, extended
-            ))
     from hypothesis.searchstrategy.recursive import RecursiveStrategy
     return RecursiveStrategy(base, extend, max_leaves)
 
@@ -857,8 +907,8 @@ def composite(f):
 
 def shared(base, key=None):
     """Returns a strategy that draws a single shared value per run, drawn from
-    base. Any two shared instances with the same key will share the same
-    value, otherwise the identity of this strategy will be used. That is:
+    base. Any two shared instances with the same key will share the same value,
+    otherwise the identity of this strategy will be used. That is:
 
     >>> x = shared(s)
     >>> y = shared(s)
@@ -868,6 +918,7 @@ def shared(base, key=None):
 
     >>> x = shared(s, key="hi")
     >>> y = shared(s, key="hi")
+
     """
     from hypothesis.searchstrategy.shared import SharedStrategy
     return SharedStrategy(base, key)
@@ -1050,7 +1101,7 @@ def check_valid_bound(value, name):
     Otherwise raises InvalidArgument.
 
     """
-    if value is None:
+    if value is None or isinstance(value, integer_types + (Rational,)):
         return
     if math.isnan(value):
         raise InvalidArgument(u'Invalid end point %s %r' % (value, name))
@@ -1058,7 +1109,9 @@ def check_valid_bound(value, name):
 
 def check_valid_size(value, name):
     """Checks that value is either unspecified, or a valid non-negative size
-    expressed as an integer/float. Otherwise raises InvalidArgument.
+    expressed as an integer/float.
+
+    Otherwise raises InvalidArgument.
 
     """
     if value is None:
